package com.om.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.ListUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.om.constant.RedisConstant;
import com.om.entity.dto.*;
import com.om.entity.po.*;
import com.om.entity.vo.UserAddVO;
import com.om.entity.vo.UserLoginVO;
import com.om.entity.vo.UserQueryPageVO;
import com.om.entity.vo.VciInfoVO;
import com.om.exception.BadReqException;
import com.om.exception.CustomerAuthenticationException;
import com.om.mapper.UserMapper;
import com.om.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.om.utils.JwtUtils;
import com.om.utils.Result;
import com.om.utils.UserContext;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 应用用户信息表 服务实现类
 * </p>
 *
 * @author bmmx
 * @since 2024-01-29
 */
@Service
@Slf4j
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements IUserService {

    @Resource
    private JwtUtils jwtUtils;

    @Resource
    private RedisTemplate redisTemplate;

    @Resource
    IUserVciService userVciService;

    @Resource
    UserMapper userMapper;

    @Resource
    IVciInfoService vciInfoService;

    @Resource
    HttpServletRequest request;

    @Resource
    private IClientService clientService;

    @Resource
    private IDistributorService distributorService;


    @Override
    public Result<UserLoginVO> login(UserLoginDTO dto) {
        //从dto中获取数据
        String username = dto.getUsername();
        String password = dto.getPassword();
        String clientNum = dto.getClientNum();
        String deviceSn = dto.getDeviceSn();
        Integer type = dto.getType();
        String appVersion = dto.getAppVersion();

        //判断数据是否为空
        if (StringUtils.isBlank(username) || StringUtils.isBlank(password) || StringUtils.isBlank(clientNum) ||
                StringUtils.isBlank(deviceSn) || type == null) {
            throw new BadReqException("数据为空！！");
        }

        //根据用户名查询用户
        User user = this.lambdaQuery()
                .eq(User::getUsername, username)
                .one();

        //判断用户是否存在
        if (BeanUtil.isEmpty(user)) {
            throw new BadReqException("该用户不存在！！");
        }


        //密码加密
        password = DigestUtils.md5DigestAsHex(password.getBytes());
        //判断密码是否正确
        if (!password.equals(user.getPassword())) {
            throw new BadReqException("密码错误");
        }
        if (user.getState()==0){
            throw new BadReqException("该用户已被禁用");
        }
        //  绑定 客户端编号 和 设备 序列号  ( 判断type是登录（0）还是激活（1） )
        if (type==1){
            user.setDeviceSn(deviceSn);
        }
        Client client = clientService.lambdaQuery()
                .eq(Client::getNumber, clientNum)
                .one();
        if (BeanUtil.isEmpty(client)){
            throw new BadReqException("该客户端不存在");
        }

        // 生成token
        Map<String, Object> claims = new HashMap<>();
        claims.put("u_id", user.getId());
        claims.put("deviceSn", deviceSn);
        claims.put("appVersion",appVersion);
        claims.put("clientNum",clientNum);
        String guid = jwtUtils.generateToken(claims);

        //封装vo返回
        UserLoginVO userLoginVO = new UserLoginVO();
        // 查询 该用户所属的 维修厂
        Integer distributorId = user.getDistributorId();//用户所属的维修厂id
        Distributor distributor = distributorService.getById(distributorId);
        BeanUtil.copyProperties(distributor,userLoginVO);

        // 查询用户 所绑定的vci设备
        List<UserVci> userVciList = userVciService.lambdaQuery()
                .eq(UserVci::getUserId, user.getId())
                .list();

        if (!BeanUtil.isEmpty(userVciList)){
            List<VciInfoVO> vciInfoVOList = new ArrayList<>();
            for (UserVci userVci : userVciList) {
                VciInfoVO vciInfoVO = new VciInfoVO();
                Integer vciInfoId = userVci.getVciInfoId();
                VciInfo vciInfo = vciInfoService.getById(vciInfoId);
                BeanUtil.copyProperties(vciInfo,vciInfoVO);

                vciInfoVOList.add(vciInfoVO);
            }

            userLoginVO.setVciInfoList(vciInfoVOList);
        }

        //设置userLoginVO
        userLoginVO.setUsername(username);
        userLoginVO.setPassword(password);
        userLoginVO.setGuid(guid);

        return Result.ok(userLoginVO);
    }

    @Override
    public Result add(UserAddVO user) {
        User addUser = new User();
        BeanUtil.copyProperties(user, addUser);
        if (addUser.getUsername() == null || addUser.getPassword() == null) {
            return Result.error();
        }
        this.save(addUser);
        //差保存维修厂的公司


        return null;
    }

    @Override
    public Result delete(Integer id) {
        if (id == null) {
            return Result.error();
        }
        boolean deleteById = this.removeById(id);
        if (deleteById == false) {
            return Result.error();
        }
        return Result.ok();
    }

    @Override
    public Result getListByUserId(Integer userId) {


        return null;
    }

    @Override
    public Result<UserQueryPageVO> getPageList(UserQueryPageDTO dto) {
        Integer pageIndex = dto.getPageIndex();
        Integer pageSize = dto.getPageSize();
        String searchUsername = dto.getSearchUsername();
        String searchPhone = dto.getSearchPhone();

        Page<User> page = this.lambdaQuery()
                .like(searchUsername != null, User::getUsername, searchUsername)
                .like(searchPhone != null, User::getTel, searchPhone)
                .orderByDesc(User::getCreateTime)
                .page(new Page<>(pageIndex, pageSize));

        UserQueryPageVO vo = new UserQueryPageVO();
        vo.setCurrent((int) page.getCurrent());
        vo.setSize((int) page.getSize());
        vo.setPages((int) page.getPages());
        vo.setTotal((int) page.getTotal());
        if (searchUsername!=null || searchPhone !=null){
            vo.setSearchCount(true);
        }
        List<User> records = page.getRecords();
        vo.setRecords(records);

        return Result.ok(vo);
    }

    @Override
    public Result getByUserId(Integer userId) {
        User user = this.getById(userId);
        if (user == null) {
            return Result.error();
        }

        return Result.ok(user);
    }

    @Override
    public Result relieve() {
        return null;
    }

    @Override
    public Result updateState(Integer id, Integer state) {
        User selectUser = getById(id);
        if (BeanUtil.isEmpty(selectUser)){
            throw new BadReqException("该用户不存在");
        }

        selectUser.setState(state);
        userMapper.update(selectUser, null);

        return Result.ok();
    }

    @Override
    public Result updateType(Integer id, Integer type) {
         User selectUser = getById(id);
         if (BeanUtil.isEmpty(selectUser)){
             throw new BadReqException("该用户不存在");
         }
        selectUser.setUserType(type);
        userMapper.update(selectUser, null);
        return Result.ok();
    }

    @Override
    public Result updateUserInfo(UserAddVO user) {
        Integer userId = UserContext.getUserId();
        User userById = getById(userId);
        BeanUtil.copyProperties(user, userById);
        userById.setId(userId);

        return Result.ok();
    }

    @Override
    public Result password(AppUserUpdatePwdDTO dto) {

        if (BeanUtil.isEmpty(dto)){
            throw new BadReqException("参数为空");
        }
        String guid = dto.getGuid();
        GuidDTO guidDTO = jwtUtils.getGuidDTOFromToken(guid);
        Integer userId = guidDTO.getUserId();

        User user = getById(userId);
        if (user==null){
            return Result.error();
        }
        user.setPassword(DigestUtils.md5DigestAsHex(dto.getNewPassword().getBytes()));
        this.updateById(user);


        return Result.ok();
    }

    @Override
    public Result register(UserLoginDTO userLoginDTO) {
        User user = new User();
        if (userLoginDTO.getUsername() == null || userLoginDTO.getPassword()==null) {
            return Result.error();
        }
        user.setUsername(userLoginDTO.getUsername());
        user.setPassword(userLoginDTO.getPassword());
        boolean save = save(user);
        if (!save){
            return Result.error();
        }
        return Result.ok();
    }

    @Override
    public Result updateUserName(AppBaseDTO dto) {
        String guid = dto.getGuid();
        GuidDTO guidDTO = jwtUtils.getGuidDTOFromToken(guid);
        Integer userId = guidDTO.getUserId();

        User user = getById(userId);
        if (user==null){
            return Result.error();
        }
        user.setUsername(dto.getUsername());
        this.update(user,null);
        return Result.ok();

    }

    @Override
    public Result logout() {

        String token = request.getHeader("token");
        token = request.getParameter("token");
        //从token中获取用户id
        Integer uId = jwtUtils.getUserIdFromToken(token);
        //判断 用户id 是否为空
        if (org.springframework.util.StringUtils.isEmpty(uId)) {
            throw new CustomerAuthenticationException("token解析失败");
        }

        //判断redis中是否存在token信息
        String tokenKey = RedisConstant.USER_TOKEN_PREFIX +uId;
        redisTemplate.delete(tokenKey);

        return Result.ok();
    }

    @Override
    public Result updateVci(AppBaseDTO dto) {

        if (BeanUtil.isEmpty(dto)){
            throw new BadReqException("参数为空");
        }
        String guid = dto.getGuid();
        GuidDTO guidDTO = jwtUtils.getGuidDTOFromToken(guid);
        Integer userId = guidDTO.getUserId();
        String vciSn = dto.getVciSn();
        VciInfo vciInfo = vciInfoService.lambdaQuery()
                .eq(VciInfo::getVciNum, vciSn)
                .one();
        if (vciInfo == null){
            return Result.error();
        }
        UserVci userVci = userVciService.lambdaQuery()
                .eq(UserVci::getUserId, userId)
                .eq(UserVci::getIsDefault, true)
                .one();
        userVci.setVciInfoId(vciInfo.getId());

        boolean update = vciInfoService.update(vciInfo, null);
        if (!update){
            return Result.error();
        }

        return Result.ok();
    }
}
